@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright © 2008-2009 Jérôme Guelfucci <jerome.guelfucci@gmail.com>
+ *  Copyright © 2008-2009 Jérôme Guelfucci <jeromeg@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 
 #include "screenshooter-global.h"
 
+#include <X11/extensions/shape.h>
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
 #include <glib.h>
@@ -36,8 +37,9 @@
 
 
 GdkPixbuf
-*screenshooter_take_screenshot   (gint                  region,
-                                  gint                  delay,
-                                  gboolean              show_mouse);
+*screenshooter_take_screenshot   (gint     region,
+                                  gint     delay,
+                                  gboolean show_mouse,
+                                  gboolean plugin);
 
 #endif
