@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright © 2008-2009 Jérôme Guelfucci <jerome.guelfucci@gmail.com>
+ *  Copyright © 2008-2009 Jérôme Guelfucci <jeromeg@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,10 +36,17 @@
 #include <libxfcegui4/libxfcegui4.h>
 
 
+GtkWidget *screenshooter_actions_dialog_new (ScreenshotData *sd);
+GtkWidget *screenshooter_region_dialog_new  (ScreenshotData *sd,
+                                             gboolean        plugin);
+gchar     *screenshooter_save_screenshot    (GdkPixbuf      *screenshot,
+                                             const gchar    *directory,
+                                             const gchar    *title,
+                                             gboolean        timestamp,
+                                             gboolean        save_dialog,
+                                             gboolean        show_preview);
+
+
+
 
-GtkWidget *screenshooter_dialog_new  (ScreenshotData *sd,
-                                      gboolean        plugin);
-gchar *screenshooter_save_screenshot (GdkPixbuf      *screenshot,
-                                      gboolean        show_save_dialog,
-                                      const gchar    *default_dir);
 #endif
