@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright © 2008-2009 Jérôme Guelfucci <jerome.guelfucci@gmail.com>
+ *  Copyright © 2008-2010 Jérôme Guelfucci <jeromeg@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,8 +40,6 @@ screenshooter_copy_to_clipboard (GdkPixbuf *screenshot)
     gtk_clipboard_get_for_display (gdk_display_get_default(), GDK_SELECTION_CLIPBOARD);
 
   gtk_clipboard_set_image (clipboard, screenshot);
-
-  gtk_clipboard_store (clipboard);
 }
 
 
@@ -54,22 +52,18 @@ screenshooter_copy_to_clipboard (GdkPixbuf *screenshot)
 void
 screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
 {
-  const gchar *home_uri = screenshooter_get_home_uri ();
+  const gchar *default_uri = screenshooter_get_xdg_image_dir_uri ();
 
   XfceRc *rc;
   gint delay = 0;
   gint region = FULLSCREEN;
   gint action = SAVE;
-  gint show_save_dialog = 1;
   gint show_mouse = 1;
-  gint close_app = 1;
-  gchar *screenshot_dir = g_strdup (home_uri);
+  gboolean timestamp = TRUE;
+  gchar *screenshot_dir = g_strdup (default_uri);
+  gchar *title = g_strdup (_("Screenshot"));
   gchar *app = g_strdup ("none");
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
   gchar *last_user = g_strdup ("");
-#endif
-#endif
 
   if (G_LIKELY (file != NULL))
     {
@@ -84,23 +78,22 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
           delay = xfce_rc_read_int_entry (rc, "delay", 0);
           region = xfce_rc_read_int_entry (rc, "region", FULLSCREEN);
           action = xfce_rc_read_int_entry (rc, "action", SAVE);
-          show_save_dialog = xfce_rc_read_int_entry (rc, "show_save_dialog", 1);
           show_mouse = xfce_rc_read_int_entry (rc, "show_mouse", 1);
-          close_app = xfce_rc_read_int_entry (rc, "close", 1);
+          timestamp = xfce_rc_read_bool_entry (rc, "timestamp", TRUE);
 
           g_free (app);
           app = g_strdup (xfce_rc_read_entry (rc, "app", "none"));
 
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
           g_free (last_user);
           last_user = g_strdup (xfce_rc_read_entry (rc, "last_user", ""));
-#endif
-#endif
 
           g_free (screenshot_dir);
           screenshot_dir =
-            g_strdup (xfce_rc_read_entry (rc, "screenshot_dir", home_uri));
+            g_strdup (xfce_rc_read_entry (rc, "screenshot_dir", default_uri));
+
+          g_free (title);
+          title =
+            g_strdup (xfce_rc_read_entry (rc, "title", _("Screenshot")));
         }
 
       TRACE ("Close the rc file");
@@ -114,16 +107,12 @@ screenshooter_read_rc_file (const gchar *file, ScreenshotData *sd)
   sd->delay = delay;
   sd->region = region;
   sd->action = action;
-  sd->show_save_dialog = show_save_dialog;
   sd->show_mouse = show_mouse;
-  sd->close = close_app;
+  sd->timestamp = timestamp;
   sd->screenshot_dir = screenshot_dir;
+  sd->title = title;
   sd->app = app;
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
   sd->last_user = last_user;
-#endif
-#endif
 }
 
 
@@ -150,20 +139,12 @@ screenshooter_write_rc_file (const gchar *file, ScreenshotData *sd)
   xfce_rc_write_int_entry (rc, "delay", sd->delay);
   xfce_rc_write_int_entry (rc, "region", sd->region);
   xfce_rc_write_int_entry (rc, "action", sd->action);
-  xfce_rc_write_int_entry (rc, "show_save_dialog", sd->show_save_dialog);
   xfce_rc_write_int_entry (rc, "show_mouse", sd->show_mouse);
-  xfce_rc_write_int_entry (rc, "close", sd->close);
   xfce_rc_write_entry (rc, "screenshot_dir", sd->screenshot_dir);
   xfce_rc_write_entry (rc, "app", sd->app);
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
   xfce_rc_write_entry (rc, "last_user", sd->last_user);
-#endif
-#endif
 
   TRACE ("Flush and close the rc file");
-
-  xfce_rc_flush (rc);
   xfce_rc_close (rc);
 }
 
@@ -183,11 +164,13 @@ screenshooter_open_screenshot (const gchar *screenshot_path, const gchar *applic
 
   TRACE ("Path was != NULL");
 
-  g_return_if_fail (!g_str_equal (application, "none"));
+  if (g_str_equal (application, "none"))
+    return;
 
   TRACE ("Application was not none");
 
-  command = g_strconcat (application, " ", screenshot_path, NULL);
+  command = g_strconcat (application, " ",
+                         "\"", screenshot_path, "\"", NULL);
 
   TRACE ("Launch the command");
 
@@ -206,7 +189,7 @@ screenshooter_open_screenshot (const gchar *screenshot_path, const gchar *applic
 
 
 
-gchar *screenshooter_get_home_uri ()
+gchar *screenshooter_get_home_uri (void)
 {
   gchar *result = NULL;
   const gchar *home_path = g_getenv ("HOME");
@@ -221,6 +204,22 @@ gchar *screenshooter_get_home_uri ()
 
 
 
+gchar *screenshooter_get_xdg_image_dir_uri (void)
+{
+  gchar *result, *tmp;
+
+  tmp = g_strdup (g_get_user_special_dir (G_USER_DIRECTORY_PICTURES));
+
+  if (tmp == NULL)
+    return screenshooter_get_home_uri ();
+
+  result = g_strconcat ("file://", tmp, NULL);
+  g_free (tmp);
+
+  return result;
+}
+
+
 gboolean screenshooter_is_remote_uri (const gchar *uri)
 {
   g_return_val_if_fail(uri != NULL, FALSE);
@@ -280,3 +279,92 @@ void screenshooter_error (const gchar *format, ...)
   g_free (message);
 }
 
+gchar *screenshooter_get_time (void)
+{
+  time_t now = time (0);
+  const struct tm *tm;
+  gchar *result, *converted;
+  gsize length;
+  gchar buffer[512];
+
+  tm = localtime (&now);
+
+  converted = g_locale_from_utf8 ("%X", -1, NULL, NULL, NULL);
+  if (G_UNLIKELY (converted == NULL))
+    converted = g_strdup ("");
+
+  length = strftime (buffer, sizeof (buffer), converted, tm);
+
+  if (G_UNLIKELY (length == 0))
+    {
+      TRACE ("Buffer is NULL");
+      buffer[0] = '\0';
+    }
+
+  result = g_locale_to_utf8 (buffer, -1, NULL, NULL, NULL);
+
+  g_free (converted);
+
+  return result;
+}
+
+gchar *screenshooter_get_date (gboolean strip_slashes)
+{
+  GDate *date = g_date_new ();
+  gchar *result;
+  gchar **split = NULL;
+  gchar buffer[512];
+  gsize length;
+
+  g_date_set_time_t (date, time (NULL));
+
+  length = g_date_strftime (buffer, sizeof (buffer), "%x", date);
+
+  if (G_UNLIKELY (length == 0))
+    {
+      TRACE ("Buffer is NULL");
+      buffer[0] = '\0';
+    }
+
+  if (strip_slashes)
+    {
+      split = g_strsplit (buffer, "/", 0);
+      result = g_strjoinv (NULL, split);
+    }
+  else
+    result = g_strdup (buffer);
+
+  g_strfreev (split);
+  g_free (date);
+
+  return result;
+}
+
+
+
+void screenshooter_open_help (void)
+{
+  GError *error_help = NULL;
+
+  /* Launch the help page and show an error dialog if there was an error. */
+  if (!g_spawn_command_line_async ("xfhelp4 xfce4-screenshooter.html",
+                                   &error_help))
+    {
+      screenshooter_error ("%s", error_help->message);
+      g_error_free (error_help);
+    }
+}
+
+
+
+gboolean
+screenshooter_f1_key (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+{
+  if (event->keyval == GDK_F1)
+    {
+      screenshooter_open_help ();
+      return TRUE;
+    }
+
+  return FALSE;
+}
