@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright © 2009 Jérôme Guelfucci <jerome.guelfucci@gmail.com>
+ *  Copyright © 2009 Jérôme Guelfucci <jeromeg@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,17 +26,17 @@
 
 #include <glib.h>
 #include <glib/gstdio.h>
-
-#include <xmlrpc-c/base.h>
-#include <xmlrpc-c/client.h>
+#include <libsoup/soup.h>
 
 #include "screenshooter-utils.h"
 #include "screenshooter-simple-job.h"
 #include "sexy-url-label.h"
 #include "katze-throbber.h"
 
-void screenshooter_upload_to_zimagez (const gchar *image_path,
-                                      gchar       *last_user);
+void screenshooter_upload_to_zimagez (const gchar  *image_path,
+                                      const gchar  *last_user,
+                                      const gchar  *title,
+                                      gchar       **new_last_user);
 
 
 #endif
