@@ -218,7 +218,7 @@ sexy_url_label_motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
 	GdkModifierType state;
 	gboolean found = FALSE;
 	GList *l;
-	int index, trailing;
+	int ind, trailing;
 	int x, y;
 	SexyUrlLabelLink *link = NULL;
 
@@ -234,13 +234,13 @@ sexy_url_label_motion_notify_event(GtkWidget *widget, GdkEventMotion *event)
 	if (pango_layout_xy_to_index(layout,
 								 (x - priv->layout_x) * PANGO_SCALE,
 								 (y - priv->layout_y) * PANGO_SCALE,
-								 &index, &trailing))
+								 &ind, &trailing))
 	{
 		for (l = priv->links; l != NULL; l = l->next)
 		{
 			link = (SexyUrlLabelLink *)l->data;
 
-			if (index >= link->start && index <= link->end)
+			if (ind >= link->start && ind <= link->end)
 			{
 				found = TRUE;
 				break;
@@ -494,7 +494,7 @@ copy_link_activate_cb(GtkMenuItem *menu_item, SexyUrlLabel *url_label)
 		return;
 
 	clipboard = gtk_widget_get_clipboard(GTK_WIDGET(url_label),
-										 GDK_SELECTION_PRIMARY);
+										 GDK_SELECTION_CLIPBOARD);
 
 	gtk_clipboard_set_text(clipboard, priv->active_link->url,
 						   strlen(priv->active_link->url));
