@@ -4,7 +4,7 @@
  *  Copyright © 2005,2006 Daniel Bobadilla Leal <dbobadil@dcc.uchile.cl>
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *  Copyright © 2006 Jani Monoses <jani@ubuntu.com>
- *  Copyright © 2008-2009 Jérôme Guelfucci <jerome.guelfucci@gmail.com>
+ *  Copyright © 2008-2010 Jérôme Guelfucci <jeromeg@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -90,6 +90,11 @@ static void
 cb_button_clicked                    (GtkWidget            *button,
                                       PluginData           *pd);
 
+static gboolean
+cb_button_scrolled                   (GtkWidget            *widget,
+                                      GdkEventScroll       *event,
+                                      PluginData *pd);
+
 static void
 cb_style_set                         (XfcePanelPlugin      *plugin,
                                       gpointer              ignored,
@@ -111,22 +116,17 @@ static gboolean
 cb_set_size (XfcePanelPlugin *plugin, int size, PluginData *pd)
 {
   GdkPixbuf *pb;
-
   int width = size - 2 - 2 * MAX (pd->button->style->xthickness,
                                     pd->button->style->ythickness);
 
   TRACE ("Get the icon from the theme");
-
   pb = xfce_themed_icon_load (SCREENSHOT_ICON_NAME, width);
 
   TRACE ("Set the new icon");
-
   gtk_image_set_from_pixbuf (GTK_IMAGE (pd->image), pb);
-
   g_object_unref (pb);
 
   TRACE ("Request size for the plugin");
-
   gtk_widget_set_size_request (GTK_WIDGET (plugin), size, size);
 
   return TRUE;
@@ -142,11 +142,13 @@ static void
 cb_free_data (XfcePanelPlugin *plugin, PluginData *pd)
 {
   if (pd->style_id)
-  	g_signal_handler_disconnect (plugin, pd->style_id);
+    g_signal_handler_disconnect (plugin, pd->style_id);
 
   pd->style_id = 0;
   g_free (pd->sd->screenshot_dir);
+  g_free (pd->sd->title);
   g_free (pd->sd->app);
+  g_free (pd->sd->last_user);
   g_free (pd->sd);
   g_free (pd);
 }
@@ -162,14 +164,45 @@ cb_button_clicked (GtkWidget *button, PluginData *pd)
 {
   /* Make the button unclickable so that the user does not press it while
 	another screenshot is in progress */
-	gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
+  gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
 
   TRACE ("Start taking the screenshot");
 
-  screenshooter_take_and_output_screenshot (pd->sd);
+  screenshooter_take_screenshot_idle (pd->sd);
 
   /* Make the panel button clickable */
-	gtk_widget_set_sensitive (GTK_WIDGET (button), TRUE);
+  gtk_widget_set_sensitive (GTK_WIDGET (button), TRUE);
+}
+
+
+
+static gboolean cb_button_scrolled (GtkWidget *widget,
+                                    GdkEventScroll *event,
+                                    PluginData *pd)
+{
+  switch (event->direction)
+    {
+      case GDK_SCROLL_UP:
+      case GDK_SCROLL_RIGHT:
+        pd->sd->region += 1;
+        if (pd->sd->region > SELECT)
+          pd->sd->region = FULLSCREEN;
+        set_panel_button_tooltip (pd);
+        gtk_widget_trigger_tooltip_query (pd->button);
+        return TRUE;
+      case GDK_SCROLL_DOWN:
+      case GDK_SCROLL_LEFT:
+        pd->sd->region -= 1;
+        if (pd->sd->region == REGION_0)
+          pd->sd->region = SELECT;
+        set_panel_button_tooltip (pd);
+        gtk_widget_trigger_tooltip_query (pd->button);
+        return TRUE;
+      default:
+        return FALSE;
+    }
+
+  return FALSE;
 }
 
 
@@ -196,7 +229,6 @@ screenshooter_plugin_read_rc_file (XfcePanelPlugin *plugin, PluginData *pd)
   gchar *rc_file = xfce_panel_plugin_lookup_rc_file (plugin);
 
   screenshooter_read_rc_file (rc_file, pd->sd);
-
   g_free (rc_file);
 }
 
@@ -209,10 +241,9 @@ pd: the associated PluginData.
 static void
 screenshooter_plugin_write_rc_file (XfcePanelPlugin *plugin, PluginData *pd)
 {
-  gchar * rc_file = xfce_panel_plugin_save_location (plugin, TRUE);
+  gchar *rc_file = xfce_panel_plugin_save_location (plugin, TRUE);
 
   screenshooter_write_rc_file (rc_file, pd->sd);
-
   g_free (rc_file);
 }
 
@@ -228,7 +259,6 @@ cb_dialog_response (GtkWidget *dlg, int response, PluginData *pd)
   if (response == GTK_RESPONSE_OK)
     {
       g_object_set_data (G_OBJECT (pd->plugin), "dialog", NULL);
-
       gtk_widget_destroy (dlg);
 
       /* Update tooltips according to the chosen option */
@@ -238,19 +268,8 @@ cb_dialog_response (GtkWidget *dlg, int response, PluginData *pd)
       xfce_panel_plugin_unblock_menu (pd->plugin);
       screenshooter_plugin_write_rc_file (pd->plugin, pd);
     }
-
-  if (response == GTK_RESPONSE_HELP)
-    {
-      GError *error_help = NULL;
-
-      /* Execute the help and show an error dialog if there was
-       * an error. */
-      if (!g_spawn_command_line_async ("xfhelp4 xfce4-screenshooter.html", &error_help))
-        {
-          screenshooter_error ("%s", error_help->message);
-          g_error_free (error_help);
-        }
-    }
+  else if (response == GTK_RESPONSE_HELP)
+    screenshooter_open_help ();
 }
 
 
@@ -262,21 +281,18 @@ cb_properties_dialog (XfcePanelPlugin *plugin, PluginData *pd)
   GtkWidget *dlg;
 
   TRACE ("Create the dialog");
-
-  dlg = screenshooter_dialog_new (pd->sd, TRUE);
+  dlg = screenshooter_region_dialog_new (pd->sd, TRUE);
 
   /* Block the menu to prevent the user from launching several dialogs at
   the same time */
-
   TRACE ("Block the menu");
-
   xfce_panel_plugin_block_menu (plugin);
 
   TRACE ("Run the dialog");
-
   g_object_set_data (G_OBJECT (plugin), "dialog", dlg);
-
   g_signal_connect (dlg, "response", G_CALLBACK (cb_dialog_response), pd);
+  g_signal_connect (dlg, "key-press-event",
+                    (GCallback) screenshooter_f1_key, NULL);
 
   gtk_widget_show (dlg);
 }
@@ -315,6 +331,7 @@ screenshooter_plugin_construct (XfcePanelPlugin *plugin)
   /* Initialise the data structs */
   PluginData *pd = g_new0 (PluginData, 1);
   ScreenshotData *sd = g_new0 (ScreenshotData, 1);
+  GFile *default_save_dir;
 
   g_thread_init (NULL);
 
@@ -328,10 +345,23 @@ screenshooter_plugin_construct (XfcePanelPlugin *plugin)
   TRACE ("Read the preferences file");
   screenshooter_plugin_read_rc_file (plugin, pd);
 
+  /* Check if the directory read from the preferences is valid */
+  default_save_dir = g_file_new_for_uri (sd->screenshot_dir);
+
+  if (G_UNLIKELY (!g_file_query_exists (default_save_dir, NULL)))
+    {
+      g_free (pd->sd->screenshot_dir);
+      pd->sd->screenshot_dir = screenshooter_get_xdg_image_dir_uri ();
+    }
+
+  g_object_unref (default_save_dir);
+
   /* We want to take only one screenshot as in CLI, but not to close the main
      loop after taking a screenshot */
-  pd->sd->cli = TRUE;
-  pd->sd->close = 0;
+  pd->sd->plugin = TRUE;
+
+  /* We want the actions dialog to be always displayed */
+  pd->sd->action_specified = FALSE;
 
   /* Create the panel button */
   TRACE ("Create the panel button");
@@ -346,28 +376,26 @@ screenshooter_plugin_construct (XfcePanelPlugin *plugin)
   set_panel_button_tooltip (pd);
 
   TRACE ("Add the button to the panel");
+  gtk_container_add (GTK_CONTAINER (plugin), pd->button);
+  xfce_panel_plugin_add_action_widget (plugin, pd->button);
   gtk_widget_show_all (pd->button);
 
-  gtk_container_add (GTK_CONTAINER (plugin), pd->button);
-
-  xfce_panel_plugin_add_action_widget (plugin, pd->button);
-
   /* Set the callbacks */
-  TRACE ("Set the clicked callback");
-  g_signal_connect (pd->button, "clicked", G_CALLBACK (cb_button_clicked), pd);
+  g_signal_connect (pd->button, "clicked",
+                    G_CALLBACK (cb_button_clicked), pd);
+  g_signal_connect (pd->button, "scroll-event",
+                    G_CALLBACK (cb_button_scrolled), pd);
+  g_signal_connect (plugin, "free-data",
+                    G_CALLBACK (cb_free_data), pd);
+  g_signal_connect (plugin, "size-changed",
+                    G_CALLBACK (cb_set_size), pd);
 
-  TRACE ("Set the free data callback");
-  g_signal_connect (plugin, "free-data", G_CALLBACK (cb_free_data), pd);
+  pd->style_id = g_signal_connect (plugin, "style-set",
+                                   G_CALLBACK (cb_style_set), pd);
 
-  TRACE ("Set the size changed callback");
-  g_signal_connect (plugin, "size-changed", G_CALLBACK (cb_set_size), pd);
-
-  TRACE ("Set the style set callback");
-  pd->style_id = g_signal_connect (plugin, "style-set", G_CALLBACK (cb_style_set), pd);
-
-  TRACE ("Set the configuration menu");
+  /* Set the configuration menu */
   xfce_panel_plugin_menu_show_configure (plugin);
-
-  g_signal_connect (plugin, "configure-plugin", G_CALLBACK (cb_properties_dialog), pd);
+  g_signal_connect (plugin, "configure-plugin",
+                    G_CALLBACK (cb_properties_dialog), pd);
 }
 XFCE_PANEL_PLUGIN_REGISTER_EXTERNAL (screenshooter_plugin_construct);
