@@ -1,6 +1,6 @@
 /*  $Id$
  *
- *  Copyright © 2008-2009 Jérôme Guelfucci <jerome.guelfucci@gmail.com>
+ *  Copyright © 2008-2010 Jérôme Guelfucci <jeromeg@xfce.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -32,13 +32,8 @@ gboolean version = FALSE;
 gboolean window = FALSE;
 gboolean region = FALSE;
 gboolean fullscreen = FALSE;
-gboolean no_save_dialog = FALSE;
-gboolean hide_mouse = FALSE;
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
+gboolean mouse = FALSE;
 gboolean upload = FALSE;
-#endif
-#endif
 gchar *screenshot_dir;
 gchar *application;
 gint delay = 0;
@@ -59,13 +54,8 @@ static GOptionEntry entries[] =
     NULL
   },
   {
-    "hide", 'h', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &no_save_dialog,
-    N_("Do not display the save dialog"),
-    NULL
-  },
-  {
-    "mouse", 'm', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &hide_mouse,
-    N_("Do not display the mouse on the screenshot"),
+    "mouse", 'm', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &mouse,
+    N_("Display the mouse on the screenshot"),
     NULL
   },
   {
@@ -85,15 +75,11 @@ static GOptionEntry entries[] =
     N_("Directory where the screenshot will be saved"),
     NULL
   },
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL  
   {
     "upload", 'u', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &upload,
     N_("Host the screenshot on ZimageZ, a free online image hosting service"),
     NULL
   },
-#endif
-#endif
   {
     "version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &version,
     N_("Version information"),
@@ -113,6 +99,28 @@ static GOptionEntry entries[] =
 
 
 
+static void
+cb_dialog_response (GtkWidget *dialog, gint response, ScreenshotData *sd)
+{
+  if (response == GTK_RESPONSE_HELP)
+    {
+      g_signal_stop_emission_by_name (dialog, "response");
+      screenshooter_open_help ();
+    }
+  else if (response == GTK_RESPONSE_OK)
+    {
+      gtk_widget_destroy (dialog);
+      g_idle_add ((GSourceFunc) screenshooter_take_screenshot_idle, sd);
+    }
+  else
+    {
+      gtk_widget_destroy (dialog);
+      gtk_main_quit ();
+    }
+}
+
+
+
 /* Main */
 
 
@@ -122,8 +130,14 @@ int main (int argc, char **argv)
   GError *cli_error = NULL;
   GFile *default_save_dir;
   const gchar *rc_file;
+  const gchar *conflict_error =
+    _("Conflicting options: --%s and --%s cannot be used at the same time.\n");
+  const gchar *ignore_error =
+    _("The --%s option is only used when --fullscreen, --window or"
+      " --region is given. It will be ignored.\n");
 
   ScreenshotData *sd = g_new0 (ScreenshotData, 1);
+  sd->plugin = FALSE;
 
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
@@ -143,21 +157,70 @@ int main (int argc, char **argv)
         }
     }
 
+  /* Exit if two region options were given */
+  if (window && fullscreen)
+    {
+      g_printerr (conflict_error, "window", "fullscreen");
+      return EXIT_FAILURE;
+    }
+  else if (window && region)
+    {
+      g_printerr (conflict_error, "window", "region");
+      return EXIT_FAILURE;
+    }
+  else if (fullscreen && region)
+    {
+      g_printerr (conflict_error, "fullscreen", "region");
+      return EXIT_FAILURE;
+    }
+
+  /* Exit if two actions options were given */
+  if (upload && (application != NULL))
+    {
+      g_printerr (conflict_error, "upload", "open");
+      return EXIT_FAILURE;
+    }
+  else if (upload && (screenshot_dir != NULL))
+    {
+      g_printerr (conflict_error, "upload", "save");
+      return EXIT_FAILURE;
+    }
+  else if ((application != NULL) && (screenshot_dir != NULL))
+    {
+      g_printerr (conflict_error, "open", "save");
+      return EXIT_FAILURE;
+    }
+
+  /* Warn that action options, mouse and delay will be ignored in 
+   * non-cli mode */
+  if ((application != NULL) && !(fullscreen || window || region))
+    g_printerr (ignore_error, "open");
+  if ((screenshot_dir != NULL)  && !(fullscreen || window || region ))
+    g_printerr (ignore_error, "save");
+  if (upload && !(fullscreen || window || region))
+    g_printerr (ignore_error, "upload");
+  if (delay && !(fullscreen || window || region))
+    g_printerr (ignore_error, "delay");
+  if (mouse && !(fullscreen || window || region))
+    g_printerr (ignore_error, "mouse");
+
   if (!g_thread_supported ())
     g_thread_init (NULL);
 
   /* Read the preferences */
-  rc_file = xfce_resource_lookup (XFCE_RESOURCE_CONFIG, "xfce4/xfce4-screenshooter");
+  rc_file = xfce_resource_save_location (XFCE_RESOURCE_CONFIG, "xfce4/xfce4-screenshooter", TRUE);
   screenshooter_read_rc_file (rc_file, sd);
 
+  /* Default to no action specified */
+  sd->action_specified = FALSE;
+
   /* Check if the directory read from the preferences is valid */
   default_save_dir = g_file_new_for_uri (sd->screenshot_dir);
 
   if (G_UNLIKELY (!g_file_query_exists (default_save_dir, NULL)))
     {
       g_free (sd->screenshot_dir);
-
-      sd->screenshot_dir = screenshooter_get_home_uri ();
+      sd->screenshot_dir = screenshooter_get_xdg_image_dir_uri ();
     }
 
   g_object_unref (default_save_dir);
@@ -173,29 +236,16 @@ int main (int argc, char **argv)
   /* If a region cli option is given, take the screenshot accordingly.*/
   if (fullscreen || window || region)
     {
-      sd->cli = TRUE;
-      sd->close = 1;
-
       /* Set the region to be captured */
       if (window)
-        {
-          sd->region = ACTIVE_WINDOW;
-        }
+        sd->region = ACTIVE_WINDOW;
       else if (fullscreen)
-        {
-          sd->region = FULLSCREEN;
-        }
+        sd->region = FULLSCREEN;
       else if (region)
-        {
-          sd->region = SELECT;
-        }
-
-      /* Wether to show the save dialog allowing to choose a filename
-       * and a save location */
-      no_save_dialog ? (sd->show_save_dialog = 0) : (sd->show_save_dialog = 1);
+        sd->region = SELECT;
 
       /* Whether to display the mouse pointer on the screenshot */
-      hide_mouse ? (sd->show_mouse = 0) : (sd->show_mouse = 1);
+      mouse ? (sd->show_mouse = 1) : (sd->show_mouse = 0);
 
       sd->delay = delay;
 
@@ -203,23 +253,21 @@ int main (int argc, char **argv)
         {
           sd->app = application;
           sd->action = OPEN;
+          sd->action_specified = TRUE;
         }
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
       else if (upload)
         {
           sd->app = g_strdup ("none");
           sd->action = UPLOAD;
+          sd->action_specified = TRUE;
         }
-#endif
-#endif
       else
         {
           sd->app = g_strdup ("none");
           sd->action = SAVE;
         }
 
-      /* If the user gave a directory name, verify that it is valid */
+      /* If the user gave a directory name, check that it is valid */
       if (screenshot_dir != NULL)
         {
           default_save_dir = g_file_new_for_commandline_arg (screenshot_dir);
@@ -228,75 +276,46 @@ int main (int argc, char **argv)
             {
               g_free (sd->screenshot_dir);
               sd->screenshot_dir = g_file_get_uri (default_save_dir);
+              sd->action_specified = TRUE;
             }
           else
-            {
-              screenshooter_error ("%s",
-                                   _("%s is not a valid directory, the default"
-                                     " directory will be used."),
+              screenshooter_error (_("%s is not a valid directory, the default"
+                                   " directory will be used."),
                                    screenshot_dir);
-            }
 
           g_object_unref (default_save_dir);
           g_free (screenshot_dir);
         }
+
+      g_idle_add ((GSourceFunc) screenshooter_take_screenshot_idle, sd);
     }
   /* Else we show a dialog which allows to set the screenshot options */
   else
     {
-      sd->cli = FALSE;
-    }
+      GtkWidget *dialog;
 
-  g_idle_add ((GSourceFunc) screenshooter_take_and_output_screenshot, sd);
+      /* Use 0 as the minimal delay */
+      if (sd->delay == 0)
+        sd->delay = 1;
+
+      /* Set the dialog up */
+      dialog = screenshooter_region_dialog_new (sd, FALSE);
+      g_signal_connect (dialog, "response",
+                        G_CALLBACK (cb_dialog_response), sd);
+      g_signal_connect (dialog, "key-press-event",
+                        G_CALLBACK (screenshooter_f1_key), NULL);
+      gtk_widget_show (dialog);
+    }
 
   gtk_main ();
 
   /* Save preferences */
-  if (!sd->cli)
-    {
-      const gchar *preferences_file =
-        xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
-                                     "xfce4/xfce4-screenshooter",
-                                     TRUE);
-
-      if (preferences_file != NULL)
-        screenshooter_write_rc_file (preferences_file, sd);
-    }
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
-  else if (sd->action == UPLOAD)
-    {
-      const gchar *preferences_file =
-        xfce_resource_save_location (XFCE_RESOURCE_CONFIG,
-                                     "xfce4/xfce4-screenshooter",
-                                     TRUE);
-      if (preferences_file != NULL)
-        {
-          XfceRc *rc;
-
-          TRACE ("Open the rc file");
-          rc = xfce_rc_simple_open (preferences_file, FALSE);
-
-          if (rc != NULL)
-            {
-                xfce_rc_write_entry (rc, "last_user", sd->last_user);
-
-                TRACE ("Flush and close the rc file");
-                xfce_rc_flush (rc);
-                xfce_rc_close (rc);
-            }
-        }
-    }
-#endif
-#endif
+  screenshooter_write_rc_file (rc_file, sd);
 
   g_free (sd->screenshot_dir);
+  g_free (sd->title);
   g_free (sd->app);
-#ifdef HAVE_XMLRPC
-#ifdef HAVE_CURL
   g_free (sd->last_user);
-#endif
-#endif
   g_free (sd);
 
   TRACE ("Ciao");
